###############
## Jan Vanhove & Raphael Berthele
## "Item-related determinants of cognate guessing in multilinguals"
## Analysis Study 2
## Contact: jan.vanhove@unifr.ch
###############

###################
### PREPARATION ###
###################

# Load Sinergia data and treat "Correct" (and "Subject") as factor
sindat = read.csv("Study2/SinergiaData.csv", header=T)
sindat$Correct = factor(sindat$Correct)
sindat$Subject = factor(sindat$Subject)

# Load Vpn (= participants) data
sinvpn = read.csv("Study2/SinergiaVpn.csv", header=T)
sinvpn$Subject = factor(sinvpn$Subject)
# Exclude participants with no self-reported French or English reading skills
sinvpn = droplevels(subset(sinvpn, FrReading > 0 & EngReading > 0))
# Add info to sindat
sindat = merge(sindat, sinvpn, by="Subject", all.y=TRUE)
summary(sindat)

# Compute percentage of correct translations per item
library(plyr) # install plyr package first: install.packages("plyr")
sin_proItem = ddply(sindat, .(Stimulus), summarise,
                    PercCorrect = length(which(Correct=="1")) / length(which(Correct=="1"|Correct=="0")))

# Load item data
sinitem = read.csv("Study2/SinergiaStimuli.csv", header=T)
# Drop profile words and merge with sin_proItem
sinitem = subset(sinitem, Status=="target")
sin_proItem = droplevels(merge(sin_proItem, sinitem, by="Stimulus", all.y=TRUE))

# Summary of % correct translation per item
summary(sin_proItem$PercCorrect); sd(sin_proItem$PercCorrect)


#####################
### RANDOM FOREST ###
#####################

# RandomForest: full model
library(party) # install party package first: install.packages("party")
sin.rf = cforest(PercCorrect ~
                   LevGer + LevGerCons + LevGerIni +
                   LevEng + LevEngCons + LevEngIni +
                   LevFre + LevFreCons + LevFreIni +
                   log.FreqGerman + log.FreqEnglish + log.FreqFrench,
                 data = sin_proItem,
                 controls=cforest_unbiased(mtry=4, ntree=1000))
# Compute conditional variable importances
# This may take some time!
sin.vi = varimp(sin.rf, conditional=TRUE)

# Combine variable importances by type
summary.sin.vi = data.frame(cbind(VarImp = sin.vi,
                                  VarName=c(
                                    "German", "German", "German", # German Levenshtein measures
                                    "English", "English", "English", # English Levenshtein measures
                                    "French", "French", "French", # French Levenshtein measures
                                    "German", "English", "French"), # Frequency measures
                                  VarType=c(
                                    "Overall Levenshtein distance","Consonantal Levenshtein distance", "Word-initial Levenshtein distance", # German
                                    "Overall Levenshtein distance","Consonantal Levenshtein distance", "Word-initial Levenshtein distance", # English
                                    "Overall Levenshtein distance","Consonantal Levenshtein distance", "Word-initial Levenshtein distance", # French
                                    "Cognate frequency (log)", "Cognate frequency (log)","Cognate frequency (log)")), # Frequency
                            stringsAsFactors = F)
summary.sin.vi$VarImp = as.numeric(summary.sin.vi$VarImp)
summary.sin.vi$VarName = factor(summary.sin.vi$VarName)
summary.sin.vi$VarType = factor(summary.sin.vi$VarType)

# Plot dotchart using rms package
library(rms) # install rms package first: install.packages("rms")
# tiff("varimp_sindat.tiff", width=8, height=7.5, units="in", res=300)
par(oma=c(0,2,0,0),
    mar=c(4,0,1,1))
dotchart2(summary.sin.vi$VarImp, labels=summary.sin.vi$VarName, groups=summary.sin.vi$VarType,
          cex = 0.9, cex.group.labels = 0.9,
          pch = 1, dotsize = 0.9,
          sort. = FALSE,
          xlab="Conditional permutation importance")
abline(v=abs(min(summary.sin.vi$VarImp)), lty=2)
# dev.off()

# Grow random forest with Germanic variables (i.e. combination of German and English)
sin2.rf = cforest(PercCorrect ~
                   LevGer + LevGerCons + LevGerIni +
                   LevEng + LevEngCons + LevEngIni +
                   LevFre + LevFreCons + LevFreIni +
                   MinLevGermanic + MinLevGermanicCons + MinLevGermanicIni + 
                   log.FreqGerman + log.FreqEnglish + log.FreqFrench + log.FreqGermanic,
                 data = sin_proItem,
                 controls=cforest_unbiased(mtry=4, ntree=1000))
# Compute conditional variable importances
sin2.vi = varimp(sin2.rf, conditional=TRUE)

# Combine variable importances into data frame
summary.sin2.vi = data.frame(cbind(VarImp = sin2.vi,
                                  VarName=c(
                                    "German", "German", "German", # German Levenshtein measures
                                    "English", "English", "English", # English Levenshtein measures
                                    "French", "French", "French", # French Levenshtein measurees
                                    "Germanic", "Germanic", "Germanic", # Germanic Levenshtein measures
                                    "German", "English", "French", "Germanic"), # Frequency measures
                                  VarType=c(
                                    "Overall Levenshtein distance","Consonantal Levenshtein distance", "Word-initial Levenshtein distance", # German
                                    "Overall Levenshtein distance","Consonantal Levenshtein distance", "Word-initial Levenshtein distance", # English
                                    "Overall Levenshtein distance","Consonantal Levenshtein distance", "Word-initial Levenshtein distance", # French
                                    "Overall Levenshtein distance","Consonantal Levenshtein distance", "Word-initial Levenshtein distance", # Germanic
                                    "Cognate frequency (log)", "Cognate frequency (log)","Cognate frequency (log)", "Cognate frequency (log)")), # Frequency
                            stringsAsFactors = F)
summary.sin2.vi$VarImp = as.numeric(summary.sin2.vi$VarImp)
summary.sin2.vi$VarName = factor(summary.sin2.vi$VarName)
summary.sin2.vi$VarType = factor(summary.sin2.vi$VarType)

# Plot dotchart using rms package
library(rms)
# tiff("varimp_sindat2.tiff", width=8, height=7.5, units="in", res=300)
par(oma=c(0,2,0,0),
    mar=c(4,0,1,1))
dotchart2(summary.sin2.vi$VarImp, labels=summary.sin2.vi$VarName, groups=summary.sin2.vi$VarType,
          cex = 0.9, cex.group.labels = 0.9,
          pch = 1, dotsize = 0.9,
          sort. = FALSE,
          xlab="Conditional permutation importance")
abline(v=abs(min(summary.sin2.vi$VarImp)), lty=2)
# dev.off()

############################
### REGRESSION MODELLING ###
############################

# Are the effects linear?
# Model MinLevGermanic and log.FreqGermanic using (nonlinear) thin plate regression splines
library(mgcv) # install mgcv first: install.packages("mgcv")
summary(gam(PercCorrect ~ s(MinLevGermanic, bs="tp") + s(log.FreqGermanic, bs="tp"),
              data = sin_proItem)) # 40.4% of variance explained using nonlinear terms
summary(lm(PercCorrect ~ MinLevGermanic + log.FreqGermanic, data=sin_proItem)) # 39% using linear terms --> hardly anything lost by modelling effects linearly


# Merge sinitem and sindat files for mixed effects modelling
sindat = merge(sindat, sinitem, by="Stimulus", all.y=TRUE)


# Load lme4 package
library(lme4) # install lme4 first: install.packages("lme4")

# Centre predictors at their means
c. = function(x) x-mean(x) # create convenience function for centring variables 
sindat$c.MinLevGermanic = sindat$MinLevGermanic - mean(sindat$MinLevGermanic)
sindat$clog.FreqGermanic = sindat$log.FreqGermanic - mean(sindat$log.FreqGermanic)

# Compute LME model
study2.mer = lmer(Correct ~ c.MinLevGermanic + clog.FreqGermanic + # fixed effects
                    (1+c.MinLevGermanic+clog.FreqGermanic|Subject) + (1|Stimulus), # random intercepts for Stimulus and Subject + correlated random slopes for MinLevGermanic and log.FreqGermanic
                  data = sindat,
                  family="binomial")
print(study2.mer, corr=F)

# Residual analysis: can any of the variables account for variance in the residuals?
# Extract the model's residuals and fit nonlinear regressions on them.
sindat$Resid = resid(study2.mer)
summary(gam(Resid ~ s(MinLevGermanic,bs="tp"), data=sindat))
summary(gam(Resid ~ s(LevGer,bs="tp"), data=sindat))
summary(gam(Resid ~ s(LevFre,bs="tp", k=5), data=sindat))
summary(gam(Resid ~ s(LevEng,bs="tp"), data=sindat))
summary(gam(Resid ~ s(log.FreqGerman,bs="tp"), data=sindat))
summary(gam(Resid ~ s(log.FreqEnglish,bs="tp"), data=sindat))
summary(gam(Resid ~ s(log.FreqFrench,bs="tp"), data=sindat))
summary(gam(Resid ~ s(LevGerCons,bs="tp"), data=sindat))
summary(gam(Resid ~ s(LevGerIni,bs="tp"), data=sindat))
summary(gam(Resid ~ s(LevEngCons,bs="tp"), data=sindat))
summary(gam(Resid ~ s(LevEngIni,bs="tp", k=3), data=sindat))
summary(gam(Resid ~ s(LevFreCons,bs="tp", k=3), data=sindat))
summary(gam(Resid ~ s(LevFreIni,bs="tp", k=3), data=sindat))
summary(gam(Resid ~ s(MinLevGermanicCons,bs="tp"), data=sindat))
summary(gam(Resid ~ s(MinLevGermanicIni,bs="tp"), data=sindat))
# No significant residual effects (not even nonlinear ones)

# Analysis of by-item random intercepts: can any of the variables account for variance in the random intercepts?
# Extract the by-item random intercepts and merge them with the item data.
sin_ranef = data.frame(ranef(study2.mer)[["Stimulus"]])
sin_ranef$Stimulus = rownames(sin_ranef)
colnames(sin_ranef) <- c("RanInt","Stimulus")
sinitem = merge(sinitem, sin_ranef, by="Stimulus",all.y=TRUE)
# Fit nonlinear regressions on them.
summary(gam(RanInt ~ s(MinLevGermanic,bs="tp"), data=sinitem))
summary(gam(RanInt ~ s(LevGer,bs="tp"), data=sinitem))
summary(gam(RanInt ~ s(LevFre,bs="tp",k=5), data=sinitem))
summary(gam(RanInt ~ s(LevEng,bs="tp"), data=sinitem))
summary(gam(RanInt ~ s(log.FreqGerman,bs="tp"), data=sinitem))
summary(gam(RanInt ~ s(log.FreqEnglish,bs="tp"), data=sinitem))
summary(gam(RanInt ~ s(log.FreqFrench,bs="tp"), data=sinitem))
summary(gam(RanInt ~ s(LevGerCons,bs="tp"), data=sinitem)) 
summary(gam(RanInt ~ s(LevGerIni,bs="tp"), data=sinitem))
summary(gam(RanInt ~ s(LevEngCons,bs="tp"), data=sinitem)); plot(gam(RanInt ~ s(LevEngCons,bs="tp"), data=sinitem)) # not in the expected direction; effect does not occur in other dataset
summary(gam(RanInt ~ s(LevEngIni,bs="tp", k=5), data=sinitem)); plot(gam(RanInt ~ s(LevEngIni,bs="tp", k=5), data=sinitem)) # not in the expected direction; effect does not occur in the other dataset
summary(gam(RanInt ~ s(LevFreCons,bs="tp", k=5), data=sinitem))
summary(gam(RanInt ~ s(LevFreIni,bs="tp", k=5), data=sinitem))
summary(gam(RanInt ~ s(MinLevGermanicCons,bs="tp"), data=sinitem))
summary(gam(RanInt ~ s(MinLevGermanicIni,bs="tp"), data=sinitem))
# No significant effects in the by-item random intercepts that occur in both datasets.